@@ -10,7 +10,8 @@ namespace Tomboy
 	{
 		static NoteTagTable instance;
 
-		public static NoteTagTable Instance {
+		public static NoteTagTable Instance 
+		{
 			get {
 				if (instance == null) 
 					instance = new NoteTagTable ();
@@ -28,6 +29,7 @@ namespace Tomboy
 			: base (ptr)
 		{
 			Console.WriteLine ("NoteTagTable Native ptr instantiation");
+			Console.WriteLine ((new System.Diagnostics.StackTrace ()).ToString ());
 		}
 		
 		void InitCommonTags () 
@@ -36,11 +38,6 @@ namespace Tomboy
 
 			// Font stylings
 
-			// This is just an empty tag we can use to get default
-			// attributes.
-			tag = new Gtk.TextTag ("normal");
-			Add (tag);
-
 			tag = new Gtk.TextTag ("centered");
 			tag.Justification = Gtk.Justification.Center;
 			Add (tag);
@@ -72,6 +69,12 @@ namespace Tomboy
 			tag.Scale = Pango.Scale.XX_Large;
 			Add (tag);
 
+			tag = new Gtk.TextTag ("related-to");
+			tag.Scale = Pango.Scale.Small;
+			tag.LeftMargin = 40;
+			tag.Editable = false;
+			Add (tag);
+
 			// Font sizes
 
 			tag = new Gtk.TextTag ("size:huge");
@@ -90,22 +93,6 @@ namespace Tomboy
 			tag.Scale = Pango.Scale.Small;
 			Add (tag);
 
-			// Font coloring
-
-			/*
-			tag = new Gtk.TextTag ("color:red");
-			tag.Foreground = "red";
-			Add (tag);
-
-			tag = new Gtk.TextTag ("color:blue");
-			tag.Foreground = "blue";
-			Add (tag);
-
-			tag = new Gtk.TextTag ("color:green");
-			tag.Foreground = "green";
-			Add (tag);
-			*/
-
 			// Lists
 
 			tag = new Gtk.TextTag ("list:bullet");
@@ -114,18 +101,6 @@ namespace Tomboy
 			tag = new Gtk.TextTag ("list:numbered");
 			Add (tag);
 
-			// Underlining
-
-			/*
-			tag = new Gtk.TextTag ("underline:single");
-			tag.Underline = Pango.Underline.Single;
-			Add (tag);
-
-			tag = new Gtk.TextTag ("underline:double");
-			tag.Underline = Pango.Underline.Double;
-			Add (tag);
-			*/
-
 			// Links
 
 			tag = new Gtk.TextTag ("link:broken");
@@ -192,6 +167,7 @@ namespace Tomboy
 	public class NoteBuffer : Gtk.TextBuffer 
 	{
 		UndoManager undo_manager;
+		Gtk.TextTagTable tag_table;
 
 		// list of Gtk.TextTags to apply on insert
 		ArrayList active_tags;
@@ -199,6 +175,7 @@ namespace Tomboy
 		public NoteBuffer (Gtk.TextTagTable tags) 
 			: base (tags)
 		{
+			tag_table = tags;
 			active_tags = new ArrayList ();
 			undo_manager = new UndoManager (this);
 
@@ -276,6 +253,10 @@ namespace Tomboy
 			Gtk.TextIter insert_start = args.Pos;
 			insert_start.BackwardChars (args.Text.Length);
 
+			foreach (Gtk.TextTag tag in insert_start.Tags) {
+				RemoveTag (tag, insert_start, args.Pos);
+			}
+
 			foreach (Gtk.TextTag tag in active_tags) {
 				ApplyTag (tag, insert_start, args.Pos);
 			}
@@ -358,6 +339,14 @@ namespace Tomboy
 
 			return word_block;
 		}
+
+		public string GetCurrentBlock (out Gtk.TextIter start, 
+					       out Gtk.TextIter end)
+		{
+			Gtk.TextIter cursor = GetIterAtMark (InsertMark);
+
+			return GetCurrentBlock (cursor, out start, out end);
+		}
 	}
 
 	public class NoteBufferArchiver
