@@ -4,6 +4,90 @@ using Mono.Posix;
 
 namespace Tomboy
 {
+	public class NoteEditor : Gtk.TextView
+	{
+		public NoteEditor (Gtk.TextBuffer buffer)
+			: base (buffer)
+		{
+			WrapMode = Gtk.WrapMode.Word;
+			LeftMargin = 8;
+			RightMargin = 8;
+			CanDefault = true;
+			DragDataReceived += OnDragDataReceived;
+			ScrollMarkOnscreen (buffer.InsertMark);
+
+			// Set Font from GConf preference
+			if ((bool) Preferences.Get (Preferences.ENABLE_CUSTOM_FONT)) {
+				string font_string = (string) Preferences.Get (Preferences.CUSTOM_FONT_FACE);
+				ModifyFont (Pango.FontDescription.FromString (font_string));
+			}
+			Preferences.SettingChanged += OnFontSettingChanged;
+
+			// Set extra editor drag targets supported (in addition
+			// to the default TextView's various text formats)...
+			Gtk.TargetList list = Gtk.Drag.DestGetTargetList (this);
+			list.Add (Gdk.Atom.Intern ("text/uri-list", false), 0, 1);
+			list.Add (Gdk.Atom.Intern ("_NETSCAPE_URL", false), 0, 1);
+		}
+
+		//
+		// Update the font based on the changed Preference dialog setting.
+		//
+		void OnFontSettingChanged (object sender, GConf.NotifyEventArgs args)
+		{
+			switch (args.Key) {
+			case Preferences.ENABLE_CUSTOM_FONT:
+				Console.WriteLine ("Switching note font {0}...", 
+						   (bool) args.Value ? "ON" : "OFF");
+
+				if ((bool) args.Value) {
+					string font_string = (string) Preferences.Get (Preferences.CUSTOM_FONT_FACE);
+					ModifyFont (Pango.FontDescription.FromString (font_string));
+				} else
+					ModifyFont (new Pango.FontDescription ());
+
+				break;
+
+			case Preferences.CUSTOM_FONT_FACE:
+				Console.WriteLine ("Switching note font to '{0}'...", 
+						   (string) args.Value);
+
+				ModifyFont (Pango.FontDescription.FromString ((string) args.Value));
+				break;
+			}
+		}
+
+		//
+		// DND Drop handling
+		//
+		void OnDragDataReceived (object sender, Gtk.DragDataReceivedArgs args)
+		{
+			// FIXME: Need to access SelectionData's target, which
+			// is unmapped in Gtk# currently, and only insert urls
+			// when the target is text/uri-list, or _NETSCAPE_URL.
+
+			UriList uri_list = new UriList (args.SelectionData);
+			if (uri_list.Count == 0)
+				return;
+
+			Gtk.TextIter cursor = Buffer.GetIterAtMark (Buffer.InsertMark);
+			bool more_than_one = false;
+
+			foreach (Uri uri in uri_list) {
+				Console.WriteLine ("Got Dropped URI: {0}", uri);
+				if (more_than_one)
+					Buffer.Insert (cursor, "\n");
+
+				if (uri.IsFile) 
+					Buffer.Insert (cursor, uri.LocalPath);
+				else
+					Buffer.Insert (cursor, uri.ToString ());
+
+				more_than_one = true;
+			}
+		}
+	}
+
 	public class NoteWindow : Gtk.Window 
 	{
 		Note note;
@@ -48,22 +132,11 @@ namespace Tomboy
 			text_menu.AttachToWidget (text_button, null);
 			text_menu.Deactivated += ReleaseTextButton;
 
-			editor = new Gtk.TextView (note.Buffer);
-			editor.WrapMode = Gtk.WrapMode.Word;
-			editor.LeftMargin = 8;
-			editor.RightMargin = 8;
-			editor.CanDefault = true;
+			// The main editor widget
+			editor = new NoteEditor (note.Buffer);
 			editor.PopulatePopup += OnPopulatePopup;
-			editor.DragDataReceived += OnDragDataReceived;
-			editor.ModifyFont (Pango.FontDescription.FromString ("Serif 11"));
-			editor.ScrollMarkOnscreen (note.Buffer.InsertMark);
 			editor.Show ();
 
-			// Set extra editor drag targets supported
-			Gtk.TargetList list = Gtk.Drag.DestGetTargetList (editor);
-			list.Add (Gdk.Atom.Intern ("text/uri-list", false), 0, 1);
-			list.Add (Gdk.Atom.Intern ("_NETSCAPE_URL", false), 0, 1);
-
 			// FIXME: I think it would be really nice to let the
 			//        window get bigger up till it grows more than
 			//        60% of the screen, and then show scrollbars. 
@@ -122,12 +195,32 @@ namespace Tomboy
 
 		protected override bool OnDeleteEvent (Gdk.Event evnt)
 		{
-			Hide ();
+			CloseWindowHandler (null, null);
 			return true;
 		}
 
+		protected override void OnHidden ()
+		{
+			base.OnHidden ();
+
+			// Workaround Gtk bug, where adding or changing Widgets
+			// while the Window is hidden causes it to be reshown at
+			// 0,0...
+			int x, y;
+			GetPosition (out x, out y);
+			Move (x, y);
+		}
+
+		// FIXME: Need to just emit a delete event, and do this work in
+		// the default delete handler, so that plugins can attach to
+		// delete event and have it always work.
 		void CloseWindowHandler (object sender, EventArgs args)
 		{
+			// Unmaximize before hiding to avoid reopening
+			// pseudo-maximized
+			if ((GdkWindow.State & Gdk.WindowState.Maximized) > 0)
+				Unmaximize ();
+
 			Hide ();
 		}
 
@@ -206,6 +299,7 @@ namespace Tomboy
 			Gtk.ImageMenuItem link = 
 				new Gtk.ImageMenuItem (Catalog.GetString ("_Link to New Note"));
 			link.Image = new Gtk.Image (Gtk.Stock.JumpTo, Gtk.IconSize.Menu);
+
 			link.Sensitive = (note.Buffer.Selection != null);
 			link.Activated += LinkToNoteActivate;
 			link.AddAccelerator ("activate",
@@ -220,7 +314,7 @@ namespace Tomboy
 			text_item.Image = new Gtk.Image (Gtk.Stock.SelectFont, Gtk.IconSize.Menu);
 			text_item.Submenu = new NoteTextMenu (accel_group, 
 							      note.Buffer, 
-							      note.Buffer.Undoer /* don't show Undo/Redo */);
+							      note.Buffer.Undoer);
 			text_item.Show ();
 
 			Gtk.ImageMenuItem find_item = 
@@ -261,38 +355,6 @@ namespace Tomboy
 			text_menu.RedoClicked (sender, args);
 		}
 
-		//
-		// DND Drop handling
-		//
-
-		void OnDragDataReceived (object sender, Gtk.DragDataReceivedArgs args)
-		{
-			// FIXME: Need to access SelectionData's target, which
-			// is unmapped in Gtk# currently, and only insert urls
-			// when the target is text/uri-list, or _NETSCAPE_URL.
-
-			UriList uri_list = new UriList (args.SelectionData);
-			if (uri_list.Count == 0)
-				return;
-
-			NoteBuffer buffer = note.Buffer;
-			Gtk.TextIter cursor = buffer.GetIterAtMark (buffer.InsertMark);
-			bool more_than_one = false;
-
-			foreach (Uri uri in uri_list) {
-				Console.WriteLine ("Got Dropped URI: {0}", uri);
-				if (more_than_one)
-					buffer.Insert (cursor, "\n");
-
-				if (uri.IsFile) 
-					buffer.Insert (cursor, uri.LocalPath);
-				else
-					buffer.Insert (cursor, uri.ToString ());
-
-				more_than_one = true;
-			}
-		}
-
 		//
 		// Toolbar
 		//
@@ -564,6 +626,10 @@ namespace Tomboy
 		Gtk.RadioMenuItem small;
 		Gtk.CheckMenuItem highlight;
 
+		// Active when the text size is indeterminable, such as when in
+		// the note's title line.
+		Gtk.RadioMenuItem hidden_no_size;
+
 		// FIXME: Tags applied to a word should hold over the space
 		// between the next word, as thats where you'll start typeing.
 		// Tags are only active -after- a character with that tag.  This
@@ -691,6 +757,9 @@ namespace Tomboy
 			small.Data ["Tag"] = "size:small";
 			small.Toggled += FontSizeClicked;
 
+			hidden_no_size = new Gtk.RadioMenuItem (small.Group, string.Empty);
+			hidden_no_size.Hide ();
+
 			RefreshState ();
 
 			Append (bold);
@@ -713,6 +782,26 @@ namespace Tomboy
 			label.UseUnderline = true;
 		}
 
+		void RefreshSizingState ()
+		{
+			Gtk.TextIter cursor = buffer.GetIterAtMark (buffer.InsertMark);
+			Gtk.TextIter selection = buffer.GetIterAtMark (buffer.SelectionBound);
+
+			// When on title line, activate the hidden menu item
+			if (cursor.Line == 0 || selection.Line == 0) {
+				hidden_no_size.Active = true;
+				return;
+			}
+
+			bool has_size = false;
+			
+			has_size |= huge.Active = buffer.IsActiveTag ("size:huge");
+			has_size |= large.Active = buffer.IsActiveTag ("size:large");
+			has_size |= small.Active = buffer.IsActiveTag ("size:small");
+
+			normal.Active = !has_size;
+		}
+
 		public void RefreshState ()
 		{
 			event_freeze = true;
@@ -724,11 +813,7 @@ namespace Tomboy
 			strikeout.Active = buffer.IsActiveTag ("strikethrough");
 			highlight.Active = buffer.IsActiveTag ("highlight");
 
-			has_size |= huge.Active = buffer.IsActiveTag ("size:huge");
-			has_size |= large.Active = buffer.IsActiveTag ("size:large");
-			has_size |= small.Active = buffer.IsActiveTag ("size:small");
-
-			normal.Active = !has_size;
+			RefreshSizingState ();
 
 			if (undo_manager != null) {
 				undo.Sensitive = undo_manager.CanUndo;
@@ -737,7 +822,8 @@ namespace Tomboy
 
 			event_freeze = false;
 		}
-		
+
+
 		// 
 		// Font-style menu item activate
 		//
@@ -770,17 +856,16 @@ namespace Tomboy
 				return;
 
 			Gtk.RadioMenuItem item = (Gtk.RadioMenuItem) sender;
-			string tag = (string) item.Data ["Tag"];
+			if (!item.Active)
+				return;
 
-			if (tag != null) {
-				// FIXME: if we want to set the font size from
-				// an accelerator, we can't rely on the state of
-				// item.Active
-				if (item.Active)
-					buffer.SetActiveTag (tag);
-				else
-					buffer.RemoveActiveTag (tag);
-			}
+			buffer.RemoveActiveTag ("size:huge");
+			buffer.RemoveActiveTag ("size:large");
+			buffer.RemoveActiveTag ("size:small");
+
+			string tag = (string) item.Data ["Tag"];
+			if (tag != null)
+				buffer.SetActiveTag (tag);
 		}
 
 		internal void UndoClicked (object sender, EventArgs args)
