@@ -1,6 +1,6 @@
 
-using DBus;
 using System;
+using System.Collections;
 
 namespace Tomboy
 {
@@ -25,6 +25,22 @@ namespace Tomboy
 			return true;
 		}
 
+		public override bool DisplayNoteWithSearch (string uri, string search)
+		{
+			Note note;
+
+			note = note_manager.FindByUri (uri);
+			if (note == null)
+				return false;
+
+			NoteFindDialog find = NoteFindDialog.GetInstance (note);
+			find.SearchText = search;
+			find.FindNextButton.Click ();
+
+			note.Window.Present ();
+			return true;
+		}
+
 		public override string FindNote (string linked_title)
 		{
 			Note note;
