@@ -1,6 +1,7 @@
 
 using System;
 using System.Collections;
+using System.Runtime.InteropServices;
 using System.Text;
 using Mono.Posix;
 
@@ -23,11 +24,11 @@ namespace Tomboy
 
 		Gtk.ListStore store;
 
+		InterruptableTimeout entry_changed_timeout;
+
 		Note current_note;
 		ArrayList current_matches;
 
-		uint changed_timeout_id = 0;
-
 		static string [] previous_searches;
 		static NoteFindDialog instance;
 		static Gdk.Pixbuf search_image;
@@ -50,10 +51,10 @@ namespace Tomboy
 				instance.search_all_notes.Sensitive = true; // allow switching
 
 				instance.UpdateResults ();
-			}
 
-			// Always reset TransientFor
-			instance.TransientFor = note.Window;
+				// FIXME: Unconnect from previous note's change events
+				instance.AddNoteChangeListener ();
+			}
 
 			return instance;
 		}
@@ -71,6 +72,9 @@ namespace Tomboy
 				instance.search_all_notes.Sensitive = false; // force it
 
 				instance.UpdateResults ();
+
+				// FIXME: Unconnect from previous manager's change events
+				instance.AddManagerChangeListeners ();
 			}
 
 			return instance;
@@ -93,16 +97,18 @@ namespace Tomboy
 			// Allow resizing if showing the results list
 			this.Resizable = search_all;
 
-			Gtk.Label label = new Gtk.Label (Catalog.GetString ("Find:"));
-
 			find_combo = new Gtk.Combo ();
 			find_combo.AllowEmpty = false;
 			find_combo.CaseSensitive = false;
-			find_combo.Entry.ActivatesDefault = true;
+			find_combo.DisableActivate ();
+			find_combo.Entry.Activated += OnEntryActivated;
 			find_combo.Entry.Changed += OnEntryChanged;
 			if (previous_searches != null)
 				find_combo.PopdownStrings = previous_searches;
 
+			Gtk.Label label = new Gtk.Label (Catalog.GetString ("_Find:"));
+			label.MnemonicWidget = find_combo.Entry;
+
 			search_all_notes = 
 				new Gtk.CheckButton (Catalog.GetString ("Search _All Notes"));
 			search_all_notes.Active = search_all;
@@ -110,7 +116,7 @@ namespace Tomboy
 			search_all_notes.Toggled += OnAllNotesToggled;
 
 			case_sensitive = 
-				new Gtk.CheckButton (Catalog.GetString ("_Case Sensitive"));
+				new Gtk.CheckButton (Catalog.GetString ("Case _Sensitive"));
 			case_sensitive.Toggled += OnCaseSensitiveToggled;
 
 			Gtk.Table widgets = new Gtk.Table (3, 2, false);
@@ -143,7 +149,7 @@ namespace Tomboy
 			if (search_all)
 				matches_window.Show ();
 
-			// Buttons at bottom: close, previous, find next
+			// Buttons at bottom: Close, Previous, Find Next
 
 			close_button = new Gtk.Button (Gtk.Stock.Close);
 			close_button.Clicked += OnCloseClicked;
@@ -176,6 +182,10 @@ namespace Tomboy
 							 Gdk.ModifierType.ControlMask,
 							 Gtk.AccelFlags.Visible);
 
+			// Set up Find Next as the default widget
+			find_next_button.CanDefault = true;
+			Default = find_next_button;
+
 			Gtk.HButtonBox button_box = new Gtk.HButtonBox ();
 			button_box.Layout = Gtk.ButtonBoxStyle.End;
 			button_box.Spacing = 8;
@@ -239,7 +249,7 @@ namespace Tomboy
 			Gtk.CellRenderer renderer;
 
 			Gtk.TreeViewColumn title = new Gtk.TreeViewColumn ();
-			title.Title = Catalog.GetString ("Search Results");
+			title.Title = Catalog.GetString ("Search _Results");
 			title.Sizing = Gtk.TreeViewColumnSizing.Autosize;
 			title.Resizable = true;
 			
@@ -285,7 +295,7 @@ namespace Tomboy
 			}
 		}
 
-		ArrayList FindMatches (NoteBuffer buffer, string [] words, bool match_case)
+		ArrayList FindMatchesInBuffer (NoteBuffer buffer, string [] words, bool match_case)
 		{
 			ArrayList matches = new ArrayList ();
 
@@ -302,7 +312,7 @@ namespace Tomboy
 				if (word == String.Empty)
 					continue;
 
-				while (true) {
+				while (true) {					
 					idx = note_text.IndexOf (word, idx);
 
 					if (idx == -1) {
@@ -336,6 +346,22 @@ namespace Tomboy
 				return matches;
 		}
 
+		bool CheckNoteHasMatch (Note note, string [] encoded_words, bool match_case)
+		{
+			string note_text = note.XmlContent;
+			if (!match_case)
+				note_text = note_text.ToLower ();
+
+			foreach (string word in encoded_words) {
+				if (note_text.IndexOf (word) > -1)
+					continue;
+				else
+					return false;
+			}
+
+			return true;
+		}
+
 		void HighlightMatches (ArrayList matches, bool highlight)
 		{
 			if (matches == null)
@@ -385,6 +411,8 @@ namespace Tomboy
 			if (current_matches == null || current_matches.Count == 0)
 				return;
 
+			Console.WriteLine ("Entering OnFindNextClicked...");
+
 			for (int i = 0; i < current_matches.Count; i++) {
 				Match match = (Match) current_matches [i];
 
@@ -400,6 +428,8 @@ namespace Tomboy
 
 			// Else wrap to first match
 			JumpToMatch ((Match) current_matches [0]);
+
+			Console.WriteLine ("Leaving OnFindNextClicked...");
 		}
 
 		void OnFindPreviousClicked (object sender, EventArgs args)
@@ -429,8 +459,6 @@ namespace Tomboy
 			if (current_matches != null)
 				HighlightMatches (current_matches, false);
 
-			this.TransientFor = null;
-
 			Hide ();
 		}
 
@@ -448,6 +476,7 @@ namespace Tomboy
 
 		void OnCaseSensitiveToggled (object sender, EventArgs args)
 		{
+			find_combo.CaseSensitive = case_sensitive.Active;
 			UpdateResults ();
 		}
 
@@ -506,7 +535,7 @@ namespace Tomboy
 			HighlightMatches (matches, true /* highlight */);
 			current_matches = matches;
 
-			// We now definately have a current_note so all
+			// We now definately have a current_note so allow
 			// switching in and out of Search All...
 			search_all_notes.Sensitive = true;
 
@@ -514,30 +543,50 @@ namespace Tomboy
 			find_prev_button.Sensitive = true;
 		}
 
-		void OnEntryChanged (object sender, EventArgs args)
+		void OnEntryActivated (object sender, EventArgs args)
 		{
-			if (changed_timeout_id != 0)
-				GLib.Source.Remove (changed_timeout_id);
+			if (entry_changed_timeout != null)
+				entry_changed_timeout.Cancel ();
 
-			changed_timeout_id = 
-				GLib.Timeout.Add (500, 
-						  new GLib.TimeoutHandler (EntryChangedTimeout));
+			// Update results
+			EntryChangedTimeout (null, null);
+
+			if (search_all_notes.Active) {
+				// Highlight the result list
+				tree.GrabFocus ();
+			} else {
+				// Jump to the first match. Activate so the
+				// button visually clicks.
+				if (find_next_button.Sensitive)
+					ActivateDefault ();
+			}
 		}
 
-		// Called in after .25 seconds of typing inactivity.  Redo the
-		// search, and update the results...
-		bool EntryChangedTimeout ()
+		void OnEntryChanged (object sender, EventArgs args)
 		{
-			CleanupMatches ();
-
-			string text = find_combo.Entry.Text;
-			if (text == null || text == String.Empty) {
-				changed_timeout_id = 0;
-				return false;
+			if (entry_changed_timeout == null) {
+				entry_changed_timeout = new InterruptableTimeout ();
+				entry_changed_timeout.Timeout += EntryChangedTimeout;
 			}
 
-			UpdateResults ();
+			entry_changed_timeout.Reset (500);
+		}
 
+		// Called in after .5 seconds of typing inactivity, or on
+		// explicit activate.  Redo the search, and update the
+		// results...
+		void EntryChangedTimeout (object sender, EventArgs args)
+		{
+			string text = find_combo.Entry.Text;
+			if (text == null || text == String.Empty)
+				return;
+
+			UpdateResults ();
+			AddToPreviousSearches (text);
+		}
+
+		void AddToPreviousSearches (string text)
+		{
 			// Update previous searches, by either creating the
 			// initial list, or adding a new term to the list, or
 			// shuffling an existing term to the top...
@@ -561,9 +610,40 @@ namespace Tomboy
 			}
 
 			find_combo.PopdownStrings = previous_searches;
+		}
 
-			changed_timeout_id = 0;
-			return false;
+		void AddManagerChangeListeners ()
+		{
+			// Update on changes to notes
+			manager.NoteDeleted += OnNoteAddOrDelete;
+			manager.NoteAdded += OnNoteAddOrDelete;
+			manager.NoteRenamed += OnNoteRenamed;
+		}
+
+		void OnNoteAddOrDelete (object sender, Note changed)
+		{
+			UpdateResults ();
+		}
+
+		void OnNoteRenamed (Note note, string old_title)
+		{
+			UpdateResults ();
+		}
+
+		void AddNoteChangeListener ()
+		{
+			current_note.Buffer.InsertText += OnInsertText;
+			current_note.Buffer.DeleteRange += OnDeleteRange;
+		}
+
+		void OnInsertText (object sender, Gtk.InsertTextArgs args)
+		{
+			UpdateResults ();
+		}
+
+		void OnDeleteRange (object sender, Gtk.DeleteRangeArgs args)
+		{
+			UpdateResults ();
 		}
 
 		void UpdateResults ()
@@ -579,6 +659,9 @@ namespace Tomboy
 
 			string [] words = text.Split (' ', '\t', '\n');
 
+			// Used for matching in the raw note XML
+			string [] encoded_words = XmlEncoder.Encode (text).Split (' ', '\t', '\n');
+
 			if (search_all_notes.Active) {
 				bool found_one = false;
 				store.Clear ();
@@ -586,10 +669,18 @@ namespace Tomboy
 				// Append in reverse chrono order (the order
 				// returned in manager.Notes)
 				foreach (Note note in manager.Notes) {
+					// Check the note's raw XML for at least
+					// one match, to avoid deserializing
+					// Buffers unnecessarily.
+					if (!CheckNoteHasMatch (note, 
+								encoded_words, 
+								case_sensitive.Active))
+						continue;
+
 					ArrayList note_matches = 
-						FindMatches (note.Buffer, 
-							     words, 
-							     case_sensitive.Active);
+						FindMatchesInBuffer (note.Buffer, 
+								     words, 
+								     case_sensitive.Active);
 
 					if (note_matches == null) 
 						continue;
@@ -621,9 +712,9 @@ namespace Tomboy
 			else if (current_note != null) {
 				Console.WriteLine ("Looking for {0}", text);
 
-				current_matches = FindMatches (current_note.Buffer, 
-							       words, 
-							       case_sensitive.Active);
+				current_matches = FindMatchesInBuffer (current_note.Buffer, 
+								       words, 
+								       case_sensitive.Active);
 				if (current_matches != null) {
 					find_next_button.Sensitive = true;
 					find_prev_button.Sensitive = true;
@@ -633,8 +724,22 @@ namespace Tomboy
 			}
 		}
 
+		[DllImport("libglib-2.0-0.dll")]
+                static extern IntPtr g_markup_escape_text (string text, int len);
+
+		// NOTE: Workaround a mapping bug in GLib.Markup.EscapeText,
+		// where the length passed to g_markup_escape_text is passed the
+		// character count, not the byte count of the string.
+		static string MarkupEscapeText (string source)
+		{
+			int len = System.Text.Encoding.UTF8.GetByteCount (source);
+			IntPtr markup = g_markup_escape_text (source, len);
+			return GLib.Marshaller.PtrToStringGFree (markup);
+		}
+
 		void AppendResultTreeView (Gtk.ListStore store, Note note, ArrayList matches)
 		{
+			string title = MarkupEscapeText (note.Title);
 			string match_cnt;
 
 			if (matches.Count > 1)
@@ -646,7 +751,7 @@ namespace Tomboy
 
 			Gtk.TreeIter iter = store.Append ();
 			store.SetValue (iter, 0 /* icon */, stock_notes);
-			store.SetValue (iter, 1 /* title */, note.Title);
+			store.SetValue (iter, 1 /* title */, title);
 			store.SetValue (iter, 2 /* match count */, match_cnt);
 			store.SetValue (iter, 3 /* note */, note);
 			store.SetValue (iter, 4 /* matches */, matches);
@@ -686,7 +791,10 @@ namespace Tomboy
 		public string SearchText
 		{
 			get { return find_combo.Entry.Text; }
-			set { find_combo.Entry.Text = value; }
+			set {
+				if (value != null)
+					find_combo.Entry.Text = value;
+			}
 		}
 	}
 }
