@@ -3,6 +3,9 @@ using System;
 using System.Collections;
 using System.Text;
 
+using System.IO;
+using System.Xml;
+
 namespace Tomboy
 {
 	public class GuiUtils 
@@ -28,6 +31,10 @@ namespace Tomboy
 		{
 			Gtk.Menu menu = (Gtk.Menu) sender;
 			menu.Popdown ();
+
+			// Unhighlight the parent
+			if (menu.AttachWidget != null)
+				menu.AttachWidget.State = Gtk.StateType.Normal;
 		}
 
 		// Place the menu underneath an arbitrary parent widget.  The
@@ -42,6 +49,15 @@ namespace Tomboy
 				    IntPtr.Zero, 
 				    (ev == null) ? 0 : ev.Button, 
 				    (ev == null) ? Gtk.Global.CurrentEventTime : ev.Time);
+
+			// Highlight the parent
+			if (menu.AttachWidget != null)
+				menu.AttachWidget.State = Gtk.StateType.Selected;
+		}
+
+		public static Gdk.Pixbuf GetIcon (string resource_name) 
+		{
+			return new Gdk.Pixbuf (null, resource_name);
 		}
 
 		public static Gdk.Pixbuf GetMiniIcon (string resource_name) 
@@ -137,16 +153,16 @@ namespace Tomboy
 
 			switch (type) {
 			case Gtk.MessageType.Error:
-				image = new Gtk.Image ("gtk-dialog-error", Gtk.IconSize.Dialog);
+				image = new Gtk.Image (Gtk.Stock.DialogError, Gtk.IconSize.Dialog);
 				break;
 			case Gtk.MessageType.Question:
-				image = new Gtk.Image ("gtk-dialog-question", Gtk.IconSize.Dialog);
+				image = new Gtk.Image (Gtk.Stock.DialogQuestion, Gtk.IconSize.Dialog);
 				break;
 			case Gtk.MessageType.Info:
-				image = new Gtk.Image ("gtk-dialog-info", Gtk.IconSize.Dialog);
+				image = new Gtk.Image (Gtk.Stock.DialogInfo, Gtk.IconSize.Dialog);
 				break;
 			case Gtk.MessageType.Warning:
-				image = new Gtk.Image ("gtk-dialog-warning", Gtk.IconSize.Dialog);
+				image = new Gtk.Image (Gtk.Stock.DialogWarning, Gtk.IconSize.Dialog);
 				break;
 			}
 
@@ -255,6 +271,10 @@ namespace Tomboy
 				if (s.EndsWith ("\r"))
 					s = s.Substring (0, s.Length - 1);
 
+				// Handle evo's broken file urls
+				if (s.StartsWith ("file:////"))
+					s = s.Replace ("file:////", "file:///");
+
 				Console.WriteLine ("uri = {0}", s);
 				try {
 					Uri uri = new Uri (s);
@@ -305,4 +325,215 @@ namespace Tomboy
 			return paths;
 		}
 	}
+
+	// Encode xml entites
+	public class XmlEncoder 
+	{
+		static StringBuilder builder;
+		static StringWriter writer;
+		static XmlTextWriter xml;
+
+		static XmlEncoder ()
+		{
+			builder = new StringBuilder ();
+			writer = new StringWriter (builder);
+			xml = new XmlTextWriter (writer);
+		}
+
+		public static string Encode (string source)
+		{
+			xml.WriteString (source);
+
+			string val = builder.ToString ();
+			builder.Length = 0;
+			return val;
+		}
+	}
+
+	// Strip xml tags
+	public class XmlDecoder 
+	{
+		static StringBuilder builder;
+
+		static XmlDecoder ()
+		{
+			builder = new StringBuilder ();
+		}
+
+		public static string Decode (string source)
+		{
+			StringReader reader = new StringReader (source);
+			XmlTextReader xml = new XmlTextReader (reader);
+			xml.Namespaces = false;
+
+			while (xml.Read ()) {
+				switch (xml.NodeType) {
+				case XmlNodeType.Text:
+				case XmlNodeType.Whitespace:
+					builder.Append (xml.Value);
+					break;
+				}
+			}
+
+			xml.Close ();
+
+			string val = builder.ToString ();
+			builder.Length = 0;
+			return val;
+		}
+	}
+
+	public class TextRange
+	{
+		Gtk.TextBuffer buffer;
+		Gtk.TextMark start_mark;
+		Gtk.TextMark end_mark;
+
+		public TextRange (Gtk.TextIter start, 
+				  Gtk.TextIter end)
+		{
+			if (start.Buffer != end.Buffer)
+				throw new Exception ("Start buffer and end buffer do not match");
+
+			buffer = start.Buffer;
+			start_mark = buffer.CreateMark (null, start, true);
+			end_mark = buffer.CreateMark (null, end, false);
+		}
+
+		public string Text
+		{
+			get { return Start.GetText (End); }
+		}
+
+		public Gtk.TextIter Start
+		{
+			get { return buffer.GetIterAtMark (start_mark); }
+			set { buffer.MoveMark (start_mark, value); }
+		}
+
+		public Gtk.TextIter End
+		{
+			get { return buffer.GetIterAtMark (end_mark); }
+			set { buffer.MoveMark (end_mark, value); }
+		}
+
+		public void Destroy ()
+		{
+			buffer.DeleteMark (start_mark);
+			buffer.DeleteMark (end_mark);
+		}
+	}
+
+	public class TextTagEnumerator : IEnumerator, IEnumerable
+	{
+		Gtk.TextBuffer buffer;
+		Gtk.TextTag tag;
+		Gtk.TextMark mark;
+		TextRange range;
+
+		public TextTagEnumerator (Gtk.TextBuffer buffer, string tag_name)
+			: this (buffer, buffer.TagTable.Lookup (tag_name))
+		{
+		}
+
+		public TextTagEnumerator (Gtk.TextBuffer buffer, Gtk.TextTag tag)
+		{
+			this.buffer = buffer;
+			this.tag = tag;
+
+			this.mark = buffer.CreateMark (null, buffer.StartIter, false);
+			this.range = new TextRange (buffer.StartIter, buffer.StartIter);
+		}
+
+		public object Current
+		{ 
+			get { return range; }
+		}
+
+		// FIXME: Mutability bugs.  multiple Links on the same line
+		// aren't getting renamed.
+		public bool MoveNext ()
+		{
+			Gtk.TextIter iter = buffer.GetIterAtMark (mark);
+
+			if (iter.Equal (buffer.EndIter)) {
+				range.Destroy ();
+				buffer.DeleteMark (mark);
+				return false;
+			}
+
+			if (!iter.ForwardToTagToggle (tag)) {
+				range.Destroy ();
+				buffer.DeleteMark (mark);
+				return false;
+			}
+
+			if (!iter.BeginsTag (tag)) {
+				buffer.MoveMark (mark, iter);
+				return MoveNext ();
+			}
+
+			range.Start = iter;
+
+			if (!iter.ForwardToTagToggle (tag)) {
+				range.Destroy ();
+				buffer.DeleteMark (mark);
+				return false;
+			}
+
+			if (!iter.EndsTag (tag)) {
+				buffer.MoveMark (mark, iter);
+				return MoveNext ();
+			}
+
+			range.End = iter;
+
+			buffer.MoveMark (mark, iter);
+
+			return true;
+		}
+
+		public void Reset ()
+		{
+			buffer.MoveMark (mark, buffer.StartIter);
+		}
+
+		public IEnumerator GetEnumerator ()
+		{
+			return this;
+		}
+	}
+
+	public class InterruptableTimeout
+	{
+		uint timeout_id;
+
+		public InterruptableTimeout ()
+		{
+		}
+
+		public void Reset (uint timeout_millis)
+		{
+			Cancel ();
+			timeout_id = GLib.Timeout.Add (timeout_millis, 
+						       new GLib.TimeoutHandler (TimeoutExpired));
+		}
+
+		public void Cancel ()
+		{
+			if (timeout_id != 0)
+				GLib.Source.Remove (timeout_id);
+		}
+
+		bool TimeoutExpired ()
+		{
+			if (Timeout != null)
+				Timeout (this, new EventArgs ());
+
+			timeout_id = 0;
+			return false;
+		}
+
+		public event EventHandler Timeout;
+	}
 }
